extern crate rand;
extern crate generic_kmeans;

use std::fmt::{Display, Formatter, Result};
use std::fs::File;
use rand::distributions::Range;
use rand::distributions::normal::Normal;
use generic_kmeans::{kmeans, Clusterable};

#[derive(PartialEq, Copy, Clone)]
struct Vector2<T> {
    pub x: T,
    pub y: T,
}

impl<T> Vector2<T> {
    pub fn new(x: T, y: T) -> Vector2<T> {
        Vector2 {
            x: x,
            y: y,
        }
    }
}

impl<T: Display> Display for Vector2<T> {
    fn fmt(&self, formatter: &mut Formatter) -> Result {
        write!(formatter, "({}, {})", self.x, self.y)
    }
}

impl Clusterable for Vector2<f64> {
    fn distance(&self, other: &Self) -> f64 {
        (self.x - other.x) * (self.x - other.y) + (self.y - other.y) * (self.y - other.y)
    }

    fn get_centroid(cluster: &Vec<Vector2<f64>>) -> Option<Vector2<f64>> {

        let len = cluster.len();

        if len == 0 {
            return None;
        }

        let mut centroid = Vector2::new(0.0, 0.0);

        for i in cluster {
            centroid.x += i.x;
            centroid.y += i.y;
        }

        centroid.x /= len as f64;
        centroid.y /= len as f64;

        Some(centroid)
    }
}

fn main() {

    use rand::distributions::IndependentSample;

    let colors = vec![
        "blue",
        "red",
        "green",
    ];

    let range = Range::new(0.0, 10.0);
    let mut rng = rand::thread_rng();
    let cluster_number = 3;

    let mut centers = vec![];
    for _ in 0..cluster_number {
        let center = Vector2::new(range.ind_sample(&mut rng), range.ind_sample(&mut rng));
        centers.push((center, Normal::new(center.x, 0.5), Normal::new(center.y, 0.5)));
    }

    let mut elements = vec![];

    for &mut (_, x_rng, y_rng) in centers.iter_mut() {

        for _ in 0..100 {
            elements.push(Vector2::new(x_rng.ind_sample(&mut rng), y_rng.ind_sample(&mut rng)));
        }

    }


    let (clusters, nb_iterations) = kmeans(
        centers.iter().map(|x| x.clone().0).collect::<Vec<_>>(), elements, 100000).ok().unwrap();

    println!("{}", nb_iterations);

    let mut output = File::create("plot/dat.dat").unwrap();

    for (cluster, color) in clusters.iter().zip(&colors) {
        for element in cluster {
            use std::io::Write;
            writeln!(output, "{} {} {}", element.x, element.y, color).unwrap();
        }
    }

    let mut center_file = File::create("plot/centers.dat").unwrap();
    for (&(center, _, _), color) in centers.iter().zip(&colors) {
        use std::io::Write;
        writeln!(center_file, "{} {} {}", center.x, center.y, color).unwrap();
    }
}
